import java.util.ArrayList;
import java.util.PriorityQueue;

/**
 * Die Klasse Kruskal fuehrt das Generieren eines 
 * minimal spanning tree nach dem
 * Algorithmus von Kruskal durch
 * 
 * @author claus albowski
 * @version Juni 2015
 */
public class Kruskal
{
    private ArrayList<Teilbaum> teilbaeume;
    private PriorityQueue<Kante> sortierte;
    private Graph graph;

    /**
     * Konstruktor für Objekte der Klasse Kruskal
     */
    public Kruskal()
    {
        graph = new Graph();
        teilbaeume = new ArrayList();
    }
    
    /**
     * statische Vorgabe des Graphen
     * sinnvoller waere Einlesen aus Datei
     */
    public void definiereGraphen() {
        String[] knotenNamen={"A","B","C","D","E","F","G","H"};
        graph.erzeugeKnoten(knotenNamen);
        String[] vonArray =
        {"A","B","B","B","C","C","C","C","D","D","E","E","E","F","F","F","G","G","G","H","H","H","H"};
        String[] nachArray =
        {"B","C","G","H","B","D","E","H","C","E","C","D","F","E","G","H","B","F","H","B","C","F","G"};
        int[] laengenArray =
        { 10, 10, 5 , 3 , 10, 6 , 4 , 8 , 6 , 3 , 4 , 3 , 10, 10, 8 , 6 , 5 , 8 , 5 , 3 , 8 , 6 , 5 };
        graph.erzeugeKanten(vonArray,nachArray,laengenArray);
        initialisiereTeilbaeume();
    }

    /**
     * Einlesen aus String
     */
    public void definiereGraphen(String liste) {
        int schritt=0;
        // 0 sucht oeffnende Klammer
        // 1 verbindet die Zeichen zum von-String
        // 2 verbindet die Zeichen zum nach-String
        // 3 verbindet die Zeichen zum laengen-String
        String von,nach,laengeString;
        ArrayList<String> vons, nachs, laengen;
        von="";
        nach="";
        laengeString="";
        vons=new ArrayList();
        nachs=new ArrayList();
        laengen=new ArrayList();
        for (int i=1; i<liste.length()-1;i++) {
            if (liste.charAt(i)=='(') {
                schritt++;
                von="";
                nach="";
                laengeString="";
                continue; }
            if ((liste.charAt(i)==' ') && (schritt==0)) {
                continue;}
            if ((liste.charAt(i)==' ') && (schritt==1)) {
                schritt=2;
                continue;}
            if ((liste.charAt(i)==' ') && (schritt==2)) {
                schritt=3;
                continue;}
            if (liste.charAt(i)==')') {
                schritt=0;
                vons.add(von);
                nachs.add(nach);
                laengen.add(laengeString);
                continue;}
            if (schritt==1) {
                von+=liste.substring(i,i+1);
                continue;
            }
            if (schritt==2) {
                nach+=liste.substring(i,i+1);
                continue;}
            if (schritt==3) {
                laengeString+=liste.substring(i,i+1);
            }
        }
        // Knotennamen herausholen
        ArrayList<String> alleKnoten = new ArrayList();
        for (String v : vons) if (!alleKnoten.contains(v)) alleKnoten.add(v);
        for (String n : nachs) if (!alleKnoten.contains(n)) alleKnoten.add(n);
        String[] knotenNamen=new String[alleKnoten.size()];
        for (int i=0; i<alleKnoten.size(); i++) knotenNamen[i]=alleKnoten.get(i);
        // vons
        String[] vonArray = new String[vons.size()];
        for (int i=0; i<vons.size(); i++) vonArray[i]=vons.get(i);
        // nachs
        String[] nachArray = new String[nachs.size()];
        for (int i=0; i<nachs.size(); i++) nachArray[i]=nachs.get(i);
        // Laengen von String in int konvertieren
        int[] laengenArray = new int[laengen.size()];
        for (int i=0; i<laengen.size();i++) laengenArray[i] = Integer.parseInt(laengen.get(i));
        //
        graph.erzeugeKnoten(knotenNamen);
        graph.erzeugeKanten(vonArray,nachArray,laengenArray);
        initialisiereTeilbaeume();
    }

    /**
     * initialisiereTeilbaeume
     */
    private void initialisiereTeilbaeume() {
        teilbaeume = new ArrayList();
        for (Knoten knoten : graph.gibKnoten()) 
            teilbaeume.add(new Teilbaum(knoten));
    }
    
    /**
     * Loesung fuer Algorithmus von Kruskal
     */
    public void minimal()
    {
        sortierte = new PriorityQueue<Kante>(graph.gibKanten());
        Kante neu;
        Teilbaum erster,zweiter,neuer;
        while (sortierte.size()>0) {
            neu = sortierte.poll();
            erster=null;
            zweiter=null;
            for (Teilbaum teilbaum : teilbaeume) {
                if (teilbaum.hatKnoten(neu.gibVon())) erster=teilbaum;
                if (teilbaum.hatKnoten(neu.gibNach())) zweiter=teilbaum;
            }
            //System.out.println(sortierte.size()+" "+teilbaeume.size());
            if (erster==null) continue;
            if (zweiter==null) continue;
            if (erster==zweiter) continue;
            neuer = new Teilbaum(erster,zweiter,neu);
            teilbaeume.add(neuer);
            teilbaeume.remove(erster);
            teilbaeume.remove(zweiter);
        }
    }
    
    /**
     * ausgabe
     */
    public void ausgabe() {
        ArrayList<Kante> alleKanten = teilbaeume.get(0).gibKanten();
        for (Kante kante : alleKanten)
        System.out.println("("+kante.gibVon().gibNamen()+" "+
                           kante.gibNach().gibNamen()+" "+
                           kante.gibLaenge()+")");
    }

}
